------------------------------------------------------------------------------
--                                                                          --
--                 GNU ADA RUN-TIME LIBRARY (GNARL) COMPONENTS              --
--                                                                          --
--                   A D A . I N T E R R U P T S . N A M E S                --
--                                                                          --
--                                  S p e c                                 --
--                                                                          --
--                             $Revision: 1.2 $                             --
--                                                                          --
--                Copyright (C) 1997, Florida State University              --
--                                                                          --
-- GNARL is free software; you can redistribute it  and/or modify it  under --
-- terms  of  the  GNU  Library General Public License  as published by the --
-- Free Software  Foundation;  either version 2, or (at  your  option)  any --
-- later  version.  GNARL is distributed  in the hope that  it will be use- --
-- ful, but but WITHOUT ANY WARRANTY;  without even the implied warranty of --
-- MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Gen- --
-- eral Library Public License  for more details.  You should have received --
-- a  copy of the GNU Library General Public License along with GNARL;  see --
-- file COPYING.LIB.  If not,  write to the  Free Software Foundation,  675 --
-- Mass Ave, Cambridge, MA 02139, USA.                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University. It is --
-- now maintained by Ada Core Technologies Inc. in cooperation with Florida --
-- State University (http://www.gnat.com).                                  --
--                                                                          --
------------------------------------------------------------------------------

--  This is the Irix (old library) version of this package
--
--  The following signals are reserved by the run time:
--
--  SIGFPE, SIGILL, SIGSEGV, SIGBUS, SIGTRAP, SIGSTOP, SIGKILL
--
--  The pragma Unreserve_All_Interrupts affects the following signal(s):
--
--  none

--  This target-dependent package spec contains names of interrupts
--  supported by the local system.

with System.OS_Interface;
--  used for names of interrupts

package Ada.Interrupts.Names is

   SIGHUP              -- hangup
      : constant Interrupt_ID := System.OS_Interface.SIGHUP;

   SIGINT              -- interrupt (rubout)
      : constant Interrupt_ID := System.OS_Interface.SIGINT;

   SIGQUIT             -- quit (ASCII FS)
      : constant Interrupt_ID := System.OS_Interface.SIGQUIT;

   SIGILL              -- illegal instruction (not reset when caught
      : constant Interrupt_ID := System.OS_Interface.SIGILL;

   SIGTRAP             -- trace trap (not reset when caught)
      : constant Interrupt_ID := System.OS_Interface.SIGTRAP;

   SIGIOT              -- IOT instruction
      : constant Interrupt_ID := System.OS_Interface.SIGIOT;

   SIGABRT             -- used by abort, replace SIGIOT in the future
      : constant Interrupt_ID := System.OS_Interface.SIGABRT;

   SIGEMT              -- EMT instruction
      : constant Interrupt_ID := System.OS_Interface.SIGEMT;

   SIGFPE              -- floating point exception
      : constant Interrupt_ID := System.OS_Interface.SIGFPE;

   SIGKILL             -- kill (cannot be caught or ignored)
      : constant Interrupt_ID := System.OS_Interface.SIGKILL;

   SIGBUS              -- bus error
      : constant Interrupt_ID := System.OS_Interface.SIGBUS;

   SIGSEGV             -- segmentation violation
      : constant Interrupt_ID := System.OS_Interface.SIGSEGV;

   SIGSYS              -- bad argument to system call
      : constant Interrupt_ID := System.OS_Interface.SIGSYS;

   SIGPIPE             -- write on a pipe with no one to read it
      : constant Interrupt_ID := System.OS_Interface.SIGPIPE;

   SIGALRM             -- alarm clock
      : constant Interrupt_ID := System.OS_Interface.SIGALRM;

   SIGTERM             -- software termination signal from kill
      : constant Interrupt_ID := System.OS_Interface.SIGTERM;

   SIGUSR1             -- user defined signal 1
      : constant Interrupt_ID := System.OS_Interface.SIGUSR1;

   SIGUSR2             -- user defined signal 2
      : constant Interrupt_ID := System.OS_Interface.SIGUSR2;

   SIGCLD              -- death of a child
      : constant Interrupt_ID := System.OS_Interface.SIGCLD;

   SIGCHLD             -- 4.3BSD's/POSIX name
      : constant Interrupt_ID := System.OS_Interface.SIGCHLD;

   SIGPWR              -- power-fail restart
      : constant Interrupt_ID := System.OS_Interface.SIGPWR;

   SIGWINCH            -- window size changes
      : constant Interrupt_ID := System.OS_Interface.SIGWINCH;

   SIGURG              -- urgent condition on IO channel
      : constant Interrupt_ID := System.OS_Interface.SIGURG;

   SIGPOLL             -- pollable event occurred
      : constant Interrupt_ID := System.OS_Interface.SIGPOLL;

   SIGIO               -- input/output possible signal
      : constant Interrupt_ID := System.OS_Interface.SIGIO;

   SIGSTOP             -- sendable stop signal not from tty
      : constant Interrupt_ID := System.OS_Interface.SIGSTOP;

   SIGTSTP             -- stop signal from tty
      : constant Interrupt_ID := System.OS_Interface.SIGTSTP;

   SIGCONT             -- continue a stopped process
      : constant Interrupt_ID := System.OS_Interface.SIGCONT;

   SIGTTIN             -- to readers pgrp upon background tty read
      : constant Interrupt_ID := System.OS_Interface.SIGTTIN;

   SIGTTOU             -- like TTIN for output if (tp->t_local&LTOSTOP)
      : constant Interrupt_ID := System.OS_Interface.SIGTTOU;

   SIGVTALRM           -- virtual time alarm
      : constant Interrupt_ID := System.OS_Interface.SIGVTALRM;

   SIGPROF             -- profiling alarm
      : constant Interrupt_ID := System.OS_Interface.SIGPROF;

   SIGXCPU             -- Cpu time limit exceeded
      : constant Interrupt_ID := System.OS_Interface.SIGXCPU;

   SIGXFSZ             -- Filesize limit exceeded for irix4 abi only
      : constant Interrupt_ID := System.OS_Interface.SIGXFSZ;

   SIG32               -- Reserved for kernel usage
      : constant Interrupt_ID := System.OS_Interface.SIG32;

   SIGRTMIN            -- Posix 1003.1b signals
      : constant Interrupt_ID := System.OS_Interface.SIGRTMIN;

   SIGRTMAX            -- Posix 1003.1b signals
      : constant Interrupt_ID := System.OS_Interface.SIGRTMAX;

end Ada.Interrupts.Names;
