/****************************************************************************/
/*                                                                          */
/*                         GNAT COMPILER COMPONENTS                         */
/*                                                                          */
/*                               A - L I N K                                */
/*                                                                          */
/*                            $Revision: 1.3 $                             */
/*                                                                          */
/*                          C Implementation File                           */
/*                                                                          */
/*          Copyright (C) 1992-1997, Free Software Foundation, Inc.         */
/*                                                                          */
/* GNAT is free software;  you can  redistribute it  and/or modify it under */
/* terms of the  GNU General Public License as published  by the Free Soft- */
/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
/* for  more details.  You should have  received  a copy of the GNU General */
/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
/* MA 02111-1307, USA.                                                      */
/*                                                                          */
/* As a  special  exception,  if you  link  this file  with other  files to */
/* produce an executable,  this file does not by itself cause the resulting */
/* executable to be covered by the GNU General Public License. This except- */
/* ion does not  however invalidate  any other reasons  why the  executable */
/* file might be covered by the  GNU Public License.                        */
/*                                                                          */
/* GNAT was originally developed  by the GNAT team at  New York University. */
/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
/*                                                                          */
/****************************************************************************/

/*  This file contains parameterizations used by gnatlink.adb in handling   */
/*  very long linker lines in systems where there are limitations on the    */
/*  argument length when the command line is used to pass items to the      */
/*  linker                                                                  */

#include <string.h>

/*  OBJECT_FILE_OPTION denotes the system dependent linker option which     */
/*  allows object file names to be placed in a file and then passed to      */
/*  the linker.                                                             */

/*  link_max is a conservative system specific threshold (in bytes) of the  */
/*  argument length passed to the linker which will trigger a file being    */
/*  used instead of the command line directly.                              */

#if defined (__osf__)
#define OBJECT_FILE_OPTION "-Wl,-input,"
int link_max = 30000;
unsigned char objlist_file_supported = 1;

#elif defined (sgi)
#define OBJECT_FILE_OPTION "-Wl,-objectlist,"
int link_max = 15000;
unsigned char objlist_file_supported = 0;

#elif defined (hpux)
#define OBJECT_FILE_OPTION "-Wl,-c,"
int link_max = 5000;
unsigned char objlist_file_supported = 1;

#elif defined (_AIX)
#define OBJECT_FILE_OPTION "-Wl,-f,"
int link_max = 15000;
unsigned char objlist_file_supported = 1;

#else
#define OBJECT_FILE_OPTION ""
int link_max = 2147483647;
unsigned char objlist_file_supported = 0;
#endif

void
get_object_file_option_ptr (len, value)
     int *len;
     char **value;
{
  *value = OBJECT_FILE_OPTION;
  if (!*value)
    *len = 0;
  else
    *len = strlen (*value);

  return;
}
