------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                               U S A G E                                  --
--                                                                          --
--                                B o d y                                   --
--                                                                          --
--                           $Revision: 1.69 $                              --
--                                                                          --
--          Copyright (C) 1992-1998, Free Software Foundation, Inc.         --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- GNAT was originally developed  by the GNAT team at  New York University. --
-- It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). --
--                                                                          --
------------------------------------------------------------------------------

with Osint;  use Osint;
with Output; use Output;

with System.WCh_Con; use System.WCh_Con;

procedure Usage is

   procedure Write_Switch_Char (Sw : String);
   --  Output two spaces followed by default switch character followed
   --  "gnat", followed by the string given as the argument, and then
   --  enough blanks to tab to column 13, i.e. assuming Sw is not longer
   --  than 5 characters, the maximum allowed, Write_Switch_Char will
   --  always output exactly 12 characters.

   procedure Write_Switch_Char (Sw : String) is
   begin
      Write_Str ("  ");
      Write_Char (Switch_Character);
      Write_Str ("gnat");
      Write_Str (Sw);

      for J in 1 .. 12 - 3 - 4 - Sw'Length loop
         Write_Char (' ');
      end loop;
   end Write_Switch_Char;

--  Start of processing for Usage

begin
   --  Usage line

   Write_Str ("Usage: ");
   Write_Program_Name;
   Write_Char (' ');
   Write_Str ("switches sfile");
   Write_Eol;
   Write_Eol;

   --  Individual lines for switches. Write_Switch_Char outputs fourteen
   --  characters, so the remaining message is allowed to be a maximum
   --  of 65 characters to be comfortable on an 80 character device.
   --  If the Write_Str fits on one line, it is short enough!

   --  Line for -a switch

   Write_Switch_Char ("a");
   Write_Str ("Assertions enabled. Pragma Assert/Debug to be activated");
   Write_Eol;

   --  Line for -b switch

   Write_Switch_Char ("b");
   Write_Str ("Generate brief messages to stderr even if verbose mode set");
   Write_Eol;

   --  Line for -c switch

   Write_Switch_Char ("c");
   Write_Str ("Check syntax and semantics only (no code generation)");
   Write_Eol;

   --  Line for -d switch

   Write_Switch_Char ("d?");
   Write_Str ("Compiler debug option ? (a-z,A-Z,0-9), see debug.adb");
   Write_Eol;

   --  Line for -e switch

   Write_Switch_Char ("e");
   Write_Str ("Error messages generated immediately, not saved up till end");
   Write_Eol;

   --  Line for -E switch

   Write_Switch_Char ("E");
   Write_Str ("Dynamic elaboration checking mode enabled");
   Write_Eol;

   --  Line for -f switch

   Write_Switch_Char ("f");
   Write_Str ("Full errors. Verbose details, all undefined references");
   Write_Eol;

   --  Line for -g switch

   Write_Switch_Char ("g");
   Write_Str ("GNAT implementation mode (used for compiling GNAT units)");
   Write_Eol;

   --  Line for -h switch

   Write_Switch_Char ("h");
   Write_Str ("Output this usage (help) information");
   Write_Eol;

   --  Line for -i switch

   Write_Switch_Char ("i?");
   Write_Str ("Identifier char set (?=1/2/3/4/8/p/f/n/w)");
   Write_Eol;

   --  Line for -k switch

   Write_Switch_Char ("k");
   Write_Str ("Limit file names to nnn characters (k = krunch)");
   Write_Eol;

   --  Line for -l switch

   Write_Switch_Char ("l");
   Write_Str ("Output full source listing with embedded error messages");
   Write_Eol;

   --  Line for -m switch

   Write_Switch_Char ("m");
   Write_Str ("Limit number of detected errors to nnn (1-999)");
   Write_Eol;

   --  Line for -n switch

   Write_Switch_Char ("n");
   Write_Str ("Inlining of subprograms (apply pragma Inline across units)");
   Write_Eol;

   --  Line for -N switch

   Write_Switch_Char ("N");
   Write_Str ("Interunit inlining of subprograms even if no pragma Inline)");
   Write_Eol;

   --  Line for -o switch

   Write_Switch_Char ("o");
   Write_Str ("Enable overflow checking (off by default)");
   Write_Eol;

   --  Line for -O switch

   Write_Switch_Char ("O nam");
   Write_Str ("Set name of output file");
   Write_Eol;

   --  Line for -p switch

   Write_Switch_Char ("p");
   Write_Str ("Suppress all checks");
   Write_Eol;

   --  Line for -P switch

   Write_Switch_Char ("P");
   Write_Str ("Generate periodic calls to System.Polling.Poll");
   Write_Eol;

   --  Line for -q switch

   Write_Switch_Char ("q");
   Write_Str ("Don't quit, try semantics, even if parse errors");
   Write_Eol;

   --  Line for -r switch

   Write_Switch_Char ("r");
   Write_Str ("Reference manual column layout required");
   Write_Eol;

   --  Lines for -s switch

   Write_Switch_Char ("s");
   Write_Str ("Syntax check only");
   Write_Eol;

   --  Lines for -t switch

   Write_Switch_Char ("t");
   Write_Str ("Tree output file to be generated");
   Write_Eol;

   --  Line for -T switch

   Write_Switch_Char ("Tnnn");
   Write_Str ("All compiler tables start at nnn times usual starting size");
   Write_Eol;

   --  Line for -u switch

   Write_Switch_Char ("u");
   Write_Str ("List units for this compilation");
   Write_Eol;

   --  Line for -v switch

   Write_Switch_Char ("v");
   Write_Str ("Verbose mode. Full error output with source lines to stdout");
   Write_Eol;

   --  Lines for -w switch

   Write_Switch_Char ("w?");
   Write_Str ("Warning mode. (?=s/e/l/u for suppress/error/elab/undefined)");
   Write_Eol;

   --  Line for -W switch

   Write_Switch_Char ("W");
   Write_Str ("Wide character encoding method");
   Write_Str (" (?=");

   for J in WC_Encoding_Method loop
      Write_Char (WC_Encoding_Letters (J));

      if J = WC_Encoding_Method'Last then
         Write_Char (')');
      else
         Write_Char ('/');
      end if;
   end loop;

   Write_Eol;

   --  Lines for -x switch

   Write_Switch_Char ("x");
   Write_Str ("Suppress output of cross-reference information");
   Write_Eol;

   --  Lines for -z switch

   Write_Switch_Char ("z");
   Write_Str ("Distribution stub generation (r/s for receiver/sender stubs)");
   Write_Eol;

   --  Line for -83 switch

   Write_Switch_Char ("83");
   Write_Str ("Enforce Ada 83 restrictions");
   Write_Eol;

   --  Line for sfile

   Write_Str ("  sfile   Source file names");
   Write_Eol;

end Usage;
